@@ -39,15 +39,12 @@ extern "C" {
 #define TASK_FIRST_PASS_ME 2
 #define TASK_SUPERRES_RE_ME 3
 #define TASK_DG_DETECTOR_HME 4
-#define SCD_LAD 6 //number of future frames
-#define PD_WINDOW_SIZE (SCD_LAD + 2) //adding previous+current to future
 #define MAX_TPL_GROUP_SIZE 512 //enough to cover 6L gop
 
 #define MAX_TPL_EXT_GROUP_SIZE MAX_TPL_GROUP_SIZE
 #define OUT_Q_ADVANCE(h) ((h == REFERENCE_QUEUE_MAX_DEPTH - 1) ? 0 : h + 1)
 #define MIN_LAD_MG 1
-#define RC_DEFAULT_LAD_MG_MT 2 // default look ahead value for rate control in Multi-threaded mode
-#define RC_DEFAULT_LAD_MG_LP1 1 // default look ahead value for rate control in LP 1
+#define RC_DEFAULT_LAD_MG 2 // default look ahead value for rate control
 void svt_aom_assert_err(uint32_t condition, char *err_msg);
 
 #define TPL_DEP_COST_SCALE_LOG2 4
@@ -59,7 +56,9 @@ void svt_aom_assert_err(uint32_t condition, char *err_msg);
 
 #define ALT_REF_QP_THRESH 20
 // Q threshold for high precision mv.
-#define HIGH_PRECISION_MV_QTHRESH 128
+#define HIGH_PRECISION_MV_QTHRESH_0 128
+#define HIGH_PRECISION_MV_QTHRESH_1 196
+#define HIGH_PRECISION_REF_PERC_TH 50
 // Actions in the second pass: Frame and SB QP assignment and temporal filtering strenght change
 #define AOM_INTERP_EXTEND 4
 #define AOM_LEFT_TOP_MARGIN_PX(subsampling) ((AOM_BORDER_IN_PIXELS >> subsampling) - AOM_INTERP_EXTEND)
@@ -79,6 +78,7 @@ void svt_aom_assert_err(uint32_t condition, char *err_msg);
 
 #define INVALID_LUMA 256
 
+#define NEAREST_NEAR_MV_CNT 4 // 1 nearest + 3 near
 typedef struct SharpnessCtrls {
     uint8_t scene_transition;
     uint8_t tf;
@@ -116,15 +116,25 @@ typedef struct MrpCtrls {
     uint8_t base_ref_list1_count;
     uint8_t non_base_ref_list0_count;
     uint8_t non_base_ref_list1_count;
+    // Use extra reference frames in the rps list for 5L.
+    uint8_t more_5L_refs;
 
-    // Limit references to (1,1) if it's safe to do so based on avg luma
-    bool safe_limit_nref;
+    // Limit references to (1,1) if it's safe to do so based on brightness and ME ZZ sad
+    // 0:off  1:brigthness + ME ZZ sad   2:brightness only. action taken at pic level in PD
+    uint8_t safe_limit_nref;
+    // used for mode 1 of safe_limit_nref. zz sad of closest references is smaller than this th
+    // 0: feature off      non-zero-value: feature on
+    uint32_t safe_limit_zz_th;
     // Limit candidate types to LAST, BWD and LAST-BWD
     bool only_l_bwd;
     // Limit PME to ref index 0 only
     bool pme_ref0_only;
-    // Use only best references
-    bool use_best_references;
+    // Use only best references decided by me in md
+    //0:speed feature off
+    //1:use with high me distortion constraint  fast
+    //2:use with TPL constraint                 faster
+    //3:use with no constraint                  fastest
+    uint8_t use_best_references;
 
 } MrpCtrls;
 typedef struct TfControls {
@@ -183,6 +193,8 @@ typedef struct TfControls {
     // Specifies the 32x32 prediction error(after subpel) under which the subpel for the 16x16
     // block(s) is bypassed
     uint64_t pred_error_32x32_th;
+    // If true, check 8x8 blocks for TF prediction
+    bool enable_8x8_pred;
 
     // Specifies whether to exit ME after HME or not (0: perform both HME and Full-Pel search, else
     // if the HME distortion is less than me_exit_th then exit after HME(i.e. do not perform the
@@ -197,6 +209,8 @@ typedef struct TfControls {
     uint8_t subpel_early_exit;
     // Specifies whether to skip reference frame e.g. 1 = use all frames, 2 = use every other frame, 4 = use 1/4 frames, etc.
     uint8_t ref_frame_factor;
+    // Specifies whether to tune the params using qp (0: OFF, 1: ON)
+    uint8_t qp_opt;
 } TfControls;
 typedef enum GM_LEVEL {
     GM_FULL   = 0, // Exhaustive search mode.
@@ -291,9 +305,8 @@ enum {
 #define NSQ_TAB_SIZE 8
 #define NUMBER_OF_DEPTH 6
 #define NUMBER_OF_SHAPES 10
-#define TF_MAX_EXTENSION 7 // Max additional tf pics after modulation per side
-#define TF_MAX_BASE_REF_PICS_6L 8 // Max additional tf pics at each side for BASE for 6L hierarchy
-#define TF_MAX_BASE_REF_PICS_SUB_6L 8 // Max additional tf pics at each side for BASE for sub-6L hierarchy
+#define TF_MAX_EXTENSION 6 // Max additional tf pics after modulation per side
+#define TF_MAX_BASE_REF_PICS 7 // Max tf pics at each side for BASE
 #define TF_MAX_L1_REF_PICS_6L 2 // Max additional tf pics at each side for L1 for 6L hierarchy
 #define TF_MAX_L1_REF_PICS_SUB_6L 1 // Max additional tf pics at each side for L1 for sub-6L hierarchy
 //  Delta QP support
@@ -740,12 +753,7 @@ typedef enum IfsLevel {
     IFS_MDS2, // IFS @ md_stage_2()
     IFS_MDS3, // IFS @ md_stage_3()
 } IfsLevel;
-typedef enum Mds0DistortionType {
-    MDS0_SAD, // Use SAD at MDS0
-    MDS0_VAR, // Use variance at MDS0
-    MDS0_SSD, // Use SSD at MDS0
-    MDS0_DIST_TYPES
-} Mds0DistortionType;
+typedef enum DistortionType { SAD, VAR, SSD, DIST_TYPES } DistortionType;
 // Profile 0.  8-bit and 10-bit 4:2:0 and 4:0:0 only.
 // Profile 1.  8-bit and 10-bit 4:4:4
 // Profile 2.  8-bit and 10-bit 4:2:2
@@ -800,29 +808,28 @@ typedef enum ATTRIBUTE_PACKED {
 } PartitionType;
 
 #define MAX_NUM_BLOCKS_ALLOC 4421
-
 typedef enum ATTRIBUTE_PACKED {
     PART_N,
     PART_H,
     PART_V,
+    PART_H4,
+    PART_V4,
     PART_HA,
     PART_HB,
     PART_VA,
     PART_VB,
-    PART_H4,
-    PART_V4,
     PART_S
 } Part;
 
 static const PartitionType from_shape_to_part[EXT_PARTITION_TYPES] = {PARTITION_NONE,
                                                                       PARTITION_HORZ,
                                                                       PARTITION_VERT,
+                                                                      PARTITION_HORZ_4,
+                                                                      PARTITION_VERT_4,
                                                                       PARTITION_HORZ_A,
                                                                       PARTITION_HORZ_B,
                                                                       PARTITION_VERT_A,
                                                                       PARTITION_VERT_B,
-                                                                      PARTITION_HORZ_4,
-                                                                      PARTITION_VERT_4,
                                                                       PARTITION_SPLIT};
 
 static const uint8_t mi_size_wide[BlockSizeS_ALL] = {1,  1,  2,  2,  2,  4, 4, 4, 8, 8, 8,
@@ -2136,7 +2143,7 @@ typedef uint8_t EbModeType;
 #define INTRA_MODE 2
 
 #define INVALID_MODE 0xFFu
-#define SPEED_CONTROL_INIT_MOD ENC_M4;
+#define SPEED_CONTROL_INIT_MOD ENC_M5;
 typedef enum ATTRIBUTE_PACKED {
     REF_LIST_0 = 0,
     REF_LIST_1 = 1,
@@ -2227,6 +2234,13 @@ typedef enum DistCalcType
     DIST_CALC_TOTAL = 2
 } DistCalcType;
 
+typedef enum DistType
+{
+    DIST_SSD = 0,
+    DIST_SSIM = 1,
+    DIST_TOTAL = 2
+} DistType;
+
 typedef enum EbPtrType
 {
     EB_N_PTR        = 0,     // malloc'd pointer
@@ -2645,6 +2659,13 @@ typedef enum IntrabcMotionDirection
 } IntrabcMotionDirection;
 typedef struct _EbEncHandle EbEncHandle;
 typedef struct _EbThreadContext EbThreadContext;
+typedef enum {
+    // level of using SSIM based function to calculate distortion in MD
+    SSIM_LVL_0 = 0,  // default, feature off
+    SSIM_LVL_1 = 1,  // use ssim cost to find best candidate in product_full_mode_decision()
+    SSIM_LVL_2 = 2,  // use ssim cost to find best tx type in tx_type_search()
+    SSIM_LVL_3 = 3   // for both product_full_mode_decision() and tx_type_search()
+} SsimLevel;
 
 #define MAX_U32 0xFFFFFFFF
 
