@@ -38,10 +38,7 @@ static void set_restoration_unit_size(int32_t width, int32_t height, int32_t sx,
 
     int32_t s = 0;
 
-    if (width * height > 352 * 288)
-        rst[0].restoration_unit_size = RESTORATION_UNITSIZE_MAX;
-    else
-        rst[0].restoration_unit_size = (RESTORATION_UNITSIZE_MAX >> 1);
+    rst[0].restoration_unit_size = RESTORATION_UNITSIZE_MAX;
 
     rst[1].restoration_unit_size = rst[0].restoration_unit_size >> s;
     rst[2].restoration_unit_size = rst[1].restoration_unit_size;
@@ -108,9 +105,10 @@ EbErrorType svt_aom_me_sb_results_ctor(MeSbResults *obj_ptr, PictureControlSetIn
                                       &max_cand_to_alloc);
     EbInputResolution resolution;
     svt_aom_derive_input_resolution(&resolution, init_data_ptr->picture_width * init_data_ptr->picture_height);
-    uint8_t number_of_pus = svt_aom_get_enable_me_16x16(init_data_ptr->enc_mode, init_data_ptr->rtc_tune)
-        ? svt_aom_get_enable_me_8x8(init_data_ptr->enc_mode, init_data_ptr->rtc_tune) ? SQUARE_PU_COUNT
-                                                                                      : MAX_SB64_PU_COUNT_NO_8X8
+    uint8_t number_of_pus = svt_aom_get_enable_me_16x16(init_data_ptr->enc_mode)
+        ? svt_aom_get_enable_me_8x8(init_data_ptr->enc_mode, init_data_ptr->rtc_tune, resolution)
+            ? SQUARE_PU_COUNT
+            : MAX_SB64_PU_COUNT_NO_8X8
         : MAX_SB64_PU_COUNT_WO_16X16;
 
     EB_MALLOC_ARRAY(obj_ptr->me_mv_array, number_of_pus * max_ref_to_alloc);
@@ -124,11 +122,10 @@ void recon_coef_dctor(EbPtr p) {
 
     EB_DELETE(obj->recon_pic_16bit);
     EB_DELETE(obj->recon_pic);
-
-    for (uint16_t sb_index = 0; sb_index < obj->b64_total_count; ++sb_index) {
+    for (uint16_t sb_index = 0; sb_index < obj->init_b64_total_count; ++sb_index) {
         EB_DELETE(obj->quantized_coeff[sb_index]); // OMK2
     }
-    EB_DELETE_PTR_ARRAY(obj->quantized_coeff, obj->b64_total_count);
+    EB_DELETE_PTR_ARRAY(obj->quantized_coeff, obj->init_b64_total_count);
 }
 static void picture_control_set_dctor(EbPtr p) {
     PictureControlSet *obj      = (PictureControlSet *)p;
@@ -270,7 +267,45 @@ EbErrorType rtime_alloc_palette_tokens(SequenceControlSet *scs, PictureControlSe
 
     return EB_ErrorNone;
 }
+/*
+recon_coef_update_param: update the parameters in EncDecSet for changing the resolution on the fly
+*/
+EbErrorType recon_coef_update_param(EncDecSet *object_ptr, SequenceControlSet *scs) {
+    EbPictureBufferDescInitData input_pic_buf_desc_init_data;
+    Bool                        is_16bit = scs->encoder_bit_depth > 8 ? TRUE : FALSE;
+    // Init Picture Init data
+    input_pic_buf_desc_init_data.max_width          = scs->max_input_luma_width;
+    input_pic_buf_desc_init_data.max_height         = scs->max_input_luma_height;
+    input_pic_buf_desc_init_data.bit_depth          = scs->encoder_bit_depth;
+    input_pic_buf_desc_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_FULL_MASK;
+    input_pic_buf_desc_init_data.color_format       = scs->static_config.encoder_color_format;
+    uint16_t padding                                = scs->sb_size + 32;
+    if (scs->static_config.superres_mode > SUPERRES_NONE || scs->static_config.resize_mode > RESIZE_NONE) {
+        padding += scs->sb_size;
+    }
+    input_pic_buf_desc_init_data.left_padding  = padding;
+    input_pic_buf_desc_init_data.right_padding = padding;
+    input_pic_buf_desc_init_data.top_padding   = padding;
+    input_pic_buf_desc_init_data.bot_padding   = padding;
+    input_pic_buf_desc_init_data.split_mode    = FALSE;
 
+    //  Reconstructed Picture Buffer
+    if (is_16bit) {
+        svt_recon_picture_buffer_desc_update(object_ptr->recon_pic_16bit, (EbPtr)&input_pic_buf_desc_init_data);
+        svt_recon_picture_buffer_desc_update(object_ptr->recon_pic, (EbPtr)&input_pic_buf_desc_init_data);
+    } else {
+        svt_recon_picture_buffer_desc_update(object_ptr->recon_pic, (EbPtr)&input_pic_buf_desc_init_data);
+        if (scs->is_16bit_pipeline) {
+            input_pic_buf_desc_init_data.bit_depth = EB_SIXTEEN_BIT;
+            svt_recon_picture_buffer_desc_update(object_ptr->recon_pic_16bit, (EbPtr)&input_pic_buf_desc_init_data);
+        }
+    }
+
+    // SB Array
+    object_ptr->b64_total_count = scs->b64_total_count;
+
+    return EB_ErrorNone;
+}
 static EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
     PictureControlSetInitData *init_data_ptr = (PictureControlSetInitData *)object_init_data_ptr;
 
@@ -330,8 +365,9 @@ static EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data
 
     // SB Array
     // object_ptr->sb_total_count          = picture_sb_width * picture_sb_height;
-    object_ptr->b64_total_count = picture_sb_width * picture_sb_height;
-    EB_ALLOC_PTR_ARRAY(object_ptr->quantized_coeff, object_ptr->b64_total_count);
+    object_ptr->b64_total_count      = picture_sb_width * picture_sb_height;
+    object_ptr->init_b64_total_count = object_ptr->b64_total_count;
+    EB_ALLOC_PTR_ARRAY(object_ptr->quantized_coeff, object_ptr->init_b64_total_count);
 
     //object_ptr->sb_total_count_pix = all_sb;
 
@@ -346,7 +382,7 @@ static EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data
     coeff_init_data.top_padding        = 0;
     coeff_init_data.bot_padding        = 0;
     coeff_init_data.split_mode         = FALSE;
-    for (sb_index = 0; sb_index < object_ptr->b64_total_count; ++sb_index) {
+    for (sb_index = 0; sb_index < object_ptr->init_b64_total_count; ++sb_index) {
         EB_NEW(object_ptr->quantized_coeff[sb_index], //OMK2
                svt_picture_buffer_desc_ctor,
                (EbPtr)&coeff_init_data);
@@ -361,6 +397,79 @@ uint32_t svt_aom_get_out_buffer_size(uint32_t picture_width, uint32_t picture_he
     else
         return (uint32_t)(EB_OUTPUTSTREAMBUFFERSIZE_MACRO(picture_width * picture_height));
 }
+/*
+pcs_update_param: update the parameters in PictureParentControlSet for changing the resolution on the fly
+*/
+EbErrorType pcs_update_param(PictureControlSet *pcs) {
+    SequenceControlSet *scs = pcs->scs;
+    // Max/Min CU Sizes
+    const uint32_t max_blk_size = scs->super_block_size;
+    // SBs
+    uint16_t sb_index;
+    uint16_t sb_origin_x;
+    uint16_t sb_origin_y;
+
+    Bool is_16bit = scs->encoder_bit_depth > 8 ? TRUE : FALSE;
+    // Init Picture Init data
+    EbPictureBufferDescInitData coeff_buffer_desc_init_data;
+    uint16_t                    padding = scs->super_block_size + 32;
+
+    coeff_buffer_desc_init_data.max_width          = scs->max_input_luma_width;
+    coeff_buffer_desc_init_data.max_height         = scs->max_input_luma_height;
+    coeff_buffer_desc_init_data.bit_depth          = EB_SIXTEEN_BIT;
+    coeff_buffer_desc_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_FULL_MASK;
+    coeff_buffer_desc_init_data.color_format       = scs->static_config.encoder_color_format;
+
+    coeff_buffer_desc_init_data.left_padding      = padding;
+    coeff_buffer_desc_init_data.right_padding     = padding;
+    coeff_buffer_desc_init_data.top_padding       = padding;
+    coeff_buffer_desc_init_data.bot_padding       = padding;
+    coeff_buffer_desc_init_data.split_mode        = FALSE;
+    coeff_buffer_desc_init_data.is_16bit_pipeline = scs->is_16bit_pipeline;
+    if ((is_16bit) || (scs->is_16bit_pipeline)) {
+        svt_picture_buffer_desc_update(pcs->input_frame16bit, (EbPtr)&coeff_buffer_desc_init_data);
+    }
+    if (svt_aom_get_enable_restoration(scs->static_config.enc_mode,
+                                       scs->static_config.enable_restoration_filtering,
+                                       scs->input_resolution,
+                                       scs->static_config.fast_decode,
+                                       scs->static_config.qp)) {
+        set_restoration_unit_size(scs->max_input_luma_width, scs->max_input_luma_height, 1, 1, pcs->rst_info);
+    }
+    pcs->frame_width  = scs->max_input_luma_width;
+    pcs->frame_height = scs->max_input_luma_height;
+    // SB Array
+    pcs->b64_total_count = scs->b64_total_count;
+    sb_origin_x          = 0;
+    sb_origin_y          = 0;
+
+    const uint16_t picture_sb_w = (uint16_t)((scs->max_input_luma_width + scs->sb_size - 1) / scs->sb_size);
+    const uint16_t picture_sb_h = (uint16_t)((scs->max_input_luma_height + scs->sb_size - 1) / scs->sb_size);
+    const uint16_t all_sb       = picture_sb_w * picture_sb_h;
+    pcs->sb_total_count         = scs->sb_total_count;
+
+    for (sb_index = 0; sb_index < all_sb; ++sb_index) {
+        pcs->sb_ptr_array[sb_index]->org_x = (uint16_t)(sb_origin_x * max_blk_size);
+        pcs->sb_ptr_array[sb_index]->org_y = (uint16_t)(sb_origin_y * max_blk_size);
+
+        pcs->sb_ptr_array[sb_index]->index = sb_index;
+        // Increment the Order in coding order (Raster Scan Order)
+        sb_origin_y = (sb_origin_x == picture_sb_w - 1) ? sb_origin_y + 1 : sb_origin_y;
+        sb_origin_x = (sb_origin_x == picture_sb_w - 1) ? 0 : sb_origin_x + 1;
+    }
+
+    uint32_t mi_stride = picture_sb_w * (scs->sb_size >> MI_SIZE_LOG2);
+    for (uint16_t mi_h = 0; mi_h < picture_sb_h * (scs->sb_size >> MI_SIZE_LOG2); mi_h++) {
+        for (uint16_t mi_w = 0; mi_w < picture_sb_w * (scs->sb_size >> MI_SIZE_LOG2); mi_w++) {
+            uint16_t mi_grid_idx = mi_h * mi_stride + mi_w;
+            uint16_t mip_idx = (mi_h >> pcs->disallow_4x4_all_frames) * (mi_stride >> pcs->disallow_4x4_all_frames) +
+                (mi_w >> pcs->disallow_4x4_all_frames);
+            pcs->mi_grid_base[mi_grid_idx] = pcs->mip + mip_idx;
+        }
+    }
+    pcs->mi_stride = picture_sb_w * (scs->sb_size >> MI_SIZE_LOG2);
+    return EB_ErrorNone;
+}
 
 static EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object_init_data_ptr) {
     PictureControlSetInitData *init_data_ptr = (PictureControlSetInitData *)object_init_data_ptr;
@@ -388,6 +497,8 @@ static EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr
 
     uint32_t output_buffer_size   = svt_aom_get_out_buffer_size(init_data_ptr->picture_width,
                                                               init_data_ptr->picture_height);
+    object_ptr->frame_width       = init_data_ptr->picture_width;
+    object_ptr->frame_height      = init_data_ptr->picture_height;
     object_ptr->tile_row_count    = init_data_ptr->tile_row_count;
     object_ptr->tile_column_count = init_data_ptr->tile_column_count;
 
@@ -414,10 +525,12 @@ static EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr
     object_ptr->temp_lf_recon_pic_16bit           = (EbPictureBufferDesc *)NULL;
     object_ptr->temp_lf_recon_pic                 = (EbPictureBufferDesc *)NULL;
     object_ptr->scaled_input_pic                  = (EbPictureBufferDesc *)NULL;
+
     if (svt_aom_get_enable_restoration(init_data_ptr->enc_mode,
                                        init_data_ptr->static_config.enable_restoration_filtering,
                                        init_data_ptr->input_resolution,
-                                       init_data_ptr->static_config.fast_decode)) {
+                                       init_data_ptr->static_config.fast_decode,
+                                       init_data_ptr->static_config.qp)) {
         set_restoration_unit_size(
             init_data_ptr->picture_width, init_data_ptr->picture_height, 1, 1, object_ptr->rst_info);
 
@@ -449,11 +562,12 @@ static EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr
     object_ptr->temporal_layer_index = 0;
 
     // SB Array
-    object_ptr->b64_total_count = picture_sb_width * picture_sb_height;
-    EB_MALLOC_ARRAY(object_ptr->sb_intra, object_ptr->b64_total_count);
-    EB_MALLOC_ARRAY(object_ptr->sb_skip, object_ptr->b64_total_count);
-    EB_MALLOC_ARRAY(object_ptr->sb_64x64_mvp, object_ptr->b64_total_count);
-    EB_MALLOC_ARRAY(object_ptr->b64_me_qindex, object_ptr->b64_total_count);
+    object_ptr->b64_total_count      = picture_sb_width * picture_sb_height;
+    object_ptr->init_b64_total_count = object_ptr->b64_total_count;
+    EB_MALLOC_ARRAY(object_ptr->sb_intra, object_ptr->init_b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->sb_skip, object_ptr->init_b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->sb_64x64_mvp, object_ptr->init_b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->b64_me_qindex, object_ptr->init_b64_total_count);
 
     sb_origin_x = 0;
     sb_origin_y = 0;
@@ -966,14 +1080,27 @@ static EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr
 
     // If NSQ is allowed, then need a 4x4 MI grid because 8x8 NSQ shapes will require 4x4 granularity
     bool disallow_4x4 = true;
-    for (uint8_t is_base = 0; is_base <= 1; is_base++)
-        for (uint8_t is_islice = 0; is_islice <= 1; is_islice++)
-            for (uint8_t coeff_lvl = 0; coeff_lvl <= HIGH_LVL + 1; coeff_lvl++)
-                disallow_4x4 = MIN(
-                    disallow_4x4,
-                    (svt_aom_get_nsq_level(init_data_ptr->enc_mode, is_islice, is_base, coeff_lvl) == 0 ? 1 : 0));
-    for (SliceType slice_type = 0; slice_type < IDR_SLICE + 1; slice_type++)
-        disallow_4x4 = MIN(disallow_4x4, svt_aom_get_disallow_4x4(init_data_ptr->enc_mode, slice_type));
+    for (uint8_t is_base = 0; is_base <= 1; is_base++) {
+        for (uint8_t is_islice = 0; is_islice <= 1; is_islice++) {
+            for (uint8_t coeff_lvl = 0; coeff_lvl <= HIGH_LVL + 1; coeff_lvl++) {
+                for (EbInputResolution res = INPUT_SIZE_240p_RANGE; res <= INPUT_SIZE_8K_RANGE; res++) {
+                    // min QP is 1 b/c 0 is lossless and is not supported
+                    for (uint8_t qp = 1; qp <= MAX_QP_VALUE; qp++) {
+                        if (!disallow_4x4)
+                            break;
+                        disallow_4x4 = MIN(
+                            disallow_4x4,
+                            (svt_aom_get_nsq_level(init_data_ptr->enc_mode, is_base, coeff_lvl, qp, res) == 0 ? 1 : 0));
+                    }
+                }
+            }
+        }
+    }
+    for (uint8_t is_islice = 0; is_islice <= 1; is_islice++) {
+        for (uint8_t is_base = 0; is_base <= 1; is_base++) {
+            disallow_4x4 = MIN(disallow_4x4, svt_aom_get_disallow_4x4(init_data_ptr->enc_mode, is_base));
+        }
+    }
 
     object_ptr->disallow_4x4_all_frames = disallow_4x4;
 
@@ -1100,6 +1227,77 @@ static void picture_parent_control_set_dctor(EbPtr ptr) {
     EB_DELETE(obj->quarter_src_pic);
     EB_DELETE(obj->sixteenth_src_pic);
 }
+/*
+ppcs_update_param: update the parameters in PictureParentControlSet for changing the resolution on the fly
+*/
+EbErrorType ppcs_update_param(PictureParentControlSet *ppcs) {
+    EbErrorType         return_error = EB_ErrorNone;
+    SequenceControlSet *scs          = ppcs->scs;
+
+    if (ppcs->av1_cm->color_format >= EB_YUV422) {
+        EbPictureBufferDescInitData input_pic_buf_desc_init_data;
+        input_pic_buf_desc_init_data.max_width          = scs->max_input_luma_width;
+        input_pic_buf_desc_init_data.max_height         = scs->max_input_luma_height;
+        input_pic_buf_desc_init_data.bit_depth          = 8; //Should be 8bit
+        input_pic_buf_desc_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_CHROMA_MASK;
+        input_pic_buf_desc_init_data.left_padding       = scs->left_padding;
+        input_pic_buf_desc_init_data.right_padding      = scs->right_padding;
+        input_pic_buf_desc_init_data.top_padding        = scs->top_padding;
+        input_pic_buf_desc_init_data.bot_padding        = scs->bot_padding;
+        input_pic_buf_desc_init_data.color_format       = EB_YUV420; //set to 420 for MD
+        input_pic_buf_desc_init_data.split_mode         = FALSE;
+        svt_picture_buffer_desc_update(ppcs->chroma_downsampled_pic, (EbPtr)&input_pic_buf_desc_init_data);
+    }
+    // GOP
+    ppcs->b64_total_count   = scs->b64_total_count;
+    ppcs->av1_cm->mi_stride = scs->pic_width_in_b64 * (BLOCK_SIZE_64 / 4);
+
+    ppcs->av1_cm->frm_size.frame_width              = scs->max_input_luma_width - scs->max_input_pad_right;
+    ppcs->av1_cm->frm_size.frame_height             = scs->max_input_luma_height - scs->max_input_pad_bottom;
+    ppcs->av1_cm->frm_size.superres_upscaled_width  = scs->max_input_luma_width - scs->max_input_pad_right;
+    ppcs->av1_cm->frm_size.superres_upscaled_height = scs->max_input_luma_height - scs->max_input_pad_bottom;
+    ppcs->av1_cm->frm_size.superres_denominator     = SCALE_NUMERATOR;
+    ppcs->av1_cm->mi_cols                           = scs->max_input_luma_width >> MI_SIZE_LOG2;
+    ppcs->av1_cm->mi_rows                           = scs->max_input_luma_height >> MI_SIZE_LOG2;
+
+    ppcs->aligned_width  = scs->max_input_luma_width;
+    ppcs->aligned_height = scs->max_input_luma_height;
+    ppcs->frame_width    = scs->max_input_luma_width;
+    ppcs->frame_height   = scs->max_input_luma_height;
+    ppcs->render_width   = scs->max_input_luma_width;
+    ppcs->render_height  = scs->max_input_luma_height;
+
+    if (svt_aom_need_gm_ref_info(scs->static_config.enc_mode, scs->static_config.resize_mode == RESIZE_NONE)) {
+        EbPictureBufferDescInitData input_pic_buf_desc_init_data;
+        input_pic_buf_desc_init_data.max_width          = scs->max_input_luma_width >> 1;
+        input_pic_buf_desc_init_data.max_height         = scs->max_input_luma_height >> 1;
+        input_pic_buf_desc_init_data.bit_depth          = 8; //Should be 8bit
+        input_pic_buf_desc_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_LUMA_MASK;
+        input_pic_buf_desc_init_data.left_padding       = 32;
+        input_pic_buf_desc_init_data.right_padding      = 32;
+        input_pic_buf_desc_init_data.top_padding        = 32;
+        input_pic_buf_desc_init_data.bot_padding        = 32;
+        input_pic_buf_desc_init_data.color_format       = EB_YUV420;
+        input_pic_buf_desc_init_data.split_mode         = FALSE;
+
+        svt_picture_buffer_desc_update(ppcs->quarter_src_pic, (EbPtr)(&input_pic_buf_desc_init_data));
+
+        input_pic_buf_desc_init_data.max_width          = scs->max_input_luma_width >> 2;
+        input_pic_buf_desc_init_data.max_height         = scs->max_input_luma_height >> 2;
+        input_pic_buf_desc_init_data.bit_depth          = 8; //Should be 8bit
+        input_pic_buf_desc_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_LUMA_MASK;
+        input_pic_buf_desc_init_data.left_padding       = 16;
+        input_pic_buf_desc_init_data.right_padding      = 16;
+        input_pic_buf_desc_init_data.top_padding        = 16;
+        input_pic_buf_desc_init_data.bot_padding        = 16;
+        input_pic_buf_desc_init_data.color_format       = EB_YUV420;
+        input_pic_buf_desc_init_data.split_mode         = FALSE;
+
+        svt_picture_buffer_desc_update(ppcs->sixteenth_src_pic, (EbPtr)(&input_pic_buf_desc_init_data));
+    }
+
+    return return_error;
+}
 static EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr, EbPtr object_init_data_ptr) {
     PictureControlSetInitData *init_data_ptr = (PictureControlSetInitData *)object_init_data_ptr;
     EbErrorType                return_error  = EB_ErrorNone;
@@ -1157,8 +1355,7 @@ static EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *obje
             block_count = 1;
         EB_MALLOC_2D(object_ptr->variance, object_ptr->b64_total_count, block_count);
     }
-
-    if (init_data_ptr->scene_change_detection) {
+    if (init_data_ptr->calc_hist) {
         EB_ALLOC_PTR_ARRAY(object_ptr->picture_histogram, MAX_NUMBER_OF_REGIONS_IN_WIDTH);
 
         for (uint32_t region_in_picture_width_index = 0; region_in_picture_width_index < MAX_NUMBER_OF_REGIONS_IN_WIDTH;
@@ -1274,16 +1471,15 @@ static EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *obje
     EB_CREATE_MUTEX(object_ptr->pcs_total_rate_mutex);
     EbInputResolution resolution;
     svt_aom_derive_input_resolution(&resolution, init_data_ptr->picture_width * init_data_ptr->picture_height);
-    object_ptr->enable_me_16x16 = svt_aom_get_enable_me_16x16(init_data_ptr->enc_mode, init_data_ptr->rtc_tune);
+    object_ptr->enable_me_16x16 = svt_aom_get_enable_me_16x16(init_data_ptr->enc_mode);
 
     // 8x8 can only be used if 16x16 is enabled
     object_ptr->enable_me_8x8 = object_ptr->enable_me_16x16
-        ? svt_aom_get_enable_me_8x8(init_data_ptr->enc_mode, init_data_ptr->rtc_tune)
+        ? svt_aom_get_enable_me_8x8(init_data_ptr->enc_mode, init_data_ptr->rtc_tune, resolution)
         : 0;
     EB_NEW(object_ptr->dg_detector, svt_aom_dg_detector_seg_ctor);
 
-    if (svt_aom_need_gm_ref_info(
-            init_data_ptr->enc_mode, true, init_data_ptr->static_config.resize_mode == RESIZE_NONE)) {
+    if (svt_aom_need_gm_ref_info(init_data_ptr->enc_mode, init_data_ptr->static_config.resize_mode == RESIZE_NONE)) {
         EbPictureBufferDescInitData input_pic_buf_desc_init_data;
         input_pic_buf_desc_init_data.max_width          = init_data_ptr->picture_width >> 1;
         input_pic_buf_desc_init_data.max_height         = init_data_ptr->picture_height >> 1;
@@ -1316,8 +1512,7 @@ static EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *obje
 }
 static void me_dctor(EbPtr p) {
     MotionEstimationData *obj = (MotionEstimationData *)p;
-
-    EB_DELETE_PTR_ARRAY(obj->me_results, obj->b64_total_count);
+    EB_DELETE_PTR_ARRAY(obj->me_results, obj->init_b64_total_count);
     if (obj->ois_mb_results)
         EB_FREE_2D(obj->ois_mb_results);
     if (obj->tpl_stats)
@@ -1330,6 +1525,17 @@ static void me_dctor(EbPtr p) {
         EB_FREE_ARRAY(obj->tpl_sb_rdmult_scaling_factors);
     if (obj->tpl_src_stats_buffer)
         EB_FREE_ARRAY(obj->tpl_src_stats_buffer);
+    if (obj->ssim_rdmult_scaling_factors)
+        EB_FREE_ARRAY(obj->ssim_rdmult_scaling_factors);
+}
+/*
+me_update_param: update the parameters in MotionEstimationData for changing the resolution on the fly
+*/
+EbErrorType me_update_param(MotionEstimationData *me_data, SequenceControlSet *scs) {
+    EbErrorType return_error = EB_ErrorNone;
+    me_data->b64_total_count = scs->b64_total_count;
+
+    return return_error;
 }
 static EbErrorType me_ctor(MotionEstimationData *object_ptr, EbPtr object_init_data_ptr) {
     PictureControlSetInitData *init_data_ptr = (PictureControlSetInitData *)object_init_data_ptr;
@@ -1340,9 +1546,10 @@ static EbErrorType me_ctor(MotionEstimationData *object_ptr, EbPtr object_init_d
                                                   init_data_ptr->b64_size);
 
     uint16_t sb_index;
-    object_ptr->dctor           = me_dctor;
-    uint32_t sb_total_count     = picture_sb_width * picture_sb_height;
-    object_ptr->b64_total_count = sb_total_count;
+    object_ptr->dctor                = me_dctor;
+    uint32_t sb_total_count          = picture_sb_width * picture_sb_height;
+    object_ptr->b64_total_count      = sb_total_count;
+    object_ptr->init_b64_total_count = sb_total_count;
 
     EB_ALLOC_PTR_ARRAY(object_ptr->me_results, sb_total_count);
 
@@ -1355,6 +1562,12 @@ static EbErrorType me_ctor(MotionEstimationData *object_ptr, EbPtr object_init_d
         const uint16_t picture_height_in_mb          = (uint16_t)((init_data_ptr->picture_height + 15) / 16);
         uint16_t       adaptive_picture_width_in_mb  = (uint16_t)((init_data_ptr->picture_width + 15) / 16);
         uint16_t       adaptive_picture_height_in_mb = (uint16_t)((init_data_ptr->picture_height + 15) / 16);
+        if (init_data_ptr->static_config.tune == 2) {
+            EB_MALLOC_ARRAY(object_ptr->ssim_rdmult_scaling_factors,
+                            adaptive_picture_width_in_mb * adaptive_picture_height_in_mb);
+        } else {
+            object_ptr->ssim_rdmult_scaling_factors = NULL;
+        }
 
         if (init_data_ptr->tpl_synth_size == 8) {
             adaptive_picture_width_in_mb  = adaptive_picture_width_in_mb << 1;
@@ -1386,6 +1599,7 @@ static EbErrorType me_ctor(MotionEstimationData *object_ptr, EbPtr object_init_d
         object_ptr->tpl_rdmult_scaling_factors    = NULL;
         object_ptr->tpl_sb_rdmult_scaling_factors = NULL;
         object_ptr->tpl_src_stats_buffer          = NULL;
+        object_ptr->ssim_rdmult_scaling_factors   = NULL;
     }
     return return_error;
 }
@@ -1486,11 +1700,25 @@ EbErrorType sb_geom_init_pcs(SequenceControlSet *scs, PictureParentControlSet *p
         for (md_scan_block_index = 0; md_scan_block_index < max_block_count; md_scan_block_index++) {
             const BlockGeom *blk_geom = get_blk_geom_mds(md_scan_block_index);
             if (scs->over_boundary_block_mode == 1) {
-                pcs->sb_geom[sb_index].block_is_allowed[md_scan_block_index] =
-                    ((pcs->sb_geom[sb_index].org_x + blk_geom->org_x + blk_geom->bwidth / 2 < encoding_width) &&
-                     (pcs->sb_geom[sb_index].org_y + blk_geom->org_y + blk_geom->bheight / 2 < encoding_height))
-                    ? TRUE
-                    : FALSE;
+                const BlockGeom *sq_blk_geom = get_blk_geom_mds(blk_geom->sqi_mds);
+                uint8_t has_rows = (pcs->sb_geom[sb_index].org_y + sq_blk_geom->org_y + sq_blk_geom->bheight / 2 <
+                                    encoding_height);
+                uint8_t has_cols = (pcs->sb_geom[sb_index].org_x + sq_blk_geom->org_x + sq_blk_geom->bwidth / 2 <
+                                    encoding_width);
+
+                // See AV1 spec section 5.11.4 for allowable blocks
+                if (has_rows && has_cols && (pcs->sb_geom[sb_index].org_y + blk_geom->org_y < encoding_height) &&
+                    (pcs->sb_geom[sb_index].org_x + blk_geom->org_x < encoding_width)) {
+                    pcs->sb_geom[sb_index].block_is_allowed[md_scan_block_index] = 1;
+                } else if (blk_geom->shape == PART_H && has_cols &&
+                           (pcs->sb_geom[sb_index].org_y + blk_geom->org_y < encoding_height)) {
+                    pcs->sb_geom[sb_index].block_is_allowed[md_scan_block_index] = 1;
+                } else if (blk_geom->shape == PART_V && has_rows &&
+                           (pcs->sb_geom[sb_index].org_x + blk_geom->org_x < encoding_width)) {
+                    pcs->sb_geom[sb_index].block_is_allowed[md_scan_block_index] = 1;
+                } else {
+                    pcs->sb_geom[sb_index].block_is_allowed[md_scan_block_index] = 0;
+                }
             } else {
                 if (blk_geom->shape != PART_N)
                     blk_geom = get_blk_geom_mds(blk_geom->sqi_mds);
